package asd.td7;

import asd.base.ObjetAClef;

/**
 * Une classe pour l'exercice 6 de la feuille 7.
 */
public class TriFusion {

    private ObjetAClef[] T; // le tableau  trier

    /**
     * Construit un objet TriFusion pour le tableau T.
     */
    public TriFusion(ObjetAClef[] T) {
        this.T = T;
    }

    /**
     * Trie T en place par la mhode du Tri par Fusion.
     * Complexit: THETA(n.lg(n)) o n = T.length.
     */
    public void trier() {
        ObjetAClef[] tmp = new ObjetAClef[T.length];
        trierEnPlace(T,tmp,0,T.length-1);
    }

    /**
     * Trie A[g..d] en place en utilisant l'espace B[g..d].
     * Prcondition : 0 <= g,d < T.length
     * Complexit: THETA(n.lg(n)) o n = d-g+1.
     */
    private void trierEnPlace(ObjetAClef[] A, ObjetAClef[] B, int g, int d) {
        if ( g < d ) {
            int m = (g + d) / 2;
            trierVers(A,B,g,m);
            trierVers(A,B,m+1,d);
            fusion(B,A,g,m,d);
        }
    }

    /**
     * Trie A[g..d] dans B[g..d].
     * Prcondition : 0 <= g <= d < T.length
     * Complexit: THETA(n.lg(n)) o n = d-g+1.
     */
    private void trierVers(ObjetAClef[] A, ObjetAClef[] B, int g, int d) {
        if ( g == d )
            B[g] = A[g];
        else {
            int m = (g + d) / 2;
            trierEnPlace(A,B,g,m);
            trierEnPlace(A,B,m+1,d);
            fusion(A,B,g,m,d);
        }
    }

    /**
     * Fusionne A[g..m] et A[m+1..d] dans B[g..d].
     * Prcondition : 0 <= g <= m <= d < T.length et
     * A[g..m] et A[m+1..d] sont tries par ordre croissant des clefs.
     * Complexit: O(n) o n = d-g+1.
     */
    private void fusion(ObjetAClef[] A, ObjetAClef[] B, int g, int m, int d) {
        int i = g; int j = m+1; int k = g;
        while ( i <= m && j <= d )
            if ( A[i].clef() < A[j].clef() )
                B[k++] = A[i++];
            else
                B[k++] = A[j++];
        while ( i <= m )
            B[k++] = A[i++];
        while ( j <= d )
            B[k++] = A[j++];
    }
}
